
from algorithms.config import Config
from algorithms.general.utils import PeriodicThread
import cv2, io, numpy as np

class CloudPreprocessor:
    def __init__(self,config: Config) -> None:
        self.config = config
        self.thread = PeriodicThread(0.05, self.compute)
        self.image = None
        self.bytes = None
        self.is_idle = False

    def start(self):
        if self.thread:
            self.thread.start()

    def stop(self):
        if self.thread:
            self.thread.stop()

    def release(self):  
        self.stop()
        
    def update(self,image,size, is_idle):
        self.image = image
        self.size = size
        self.is_idle = is_idle
        
    def get_image_bytes(self):
        return self.bytes
    
    def compute(self):  
        if not (
            self.config.cloud.scene_text_spotting
            or self.config.cloud.image_captioning
            or self.config.cloud.image_tagging
        ):
            return
        if self.image is None:
            return
        if self.is_idle:
            return
        self.preprocess()    
        
    def preprocess(self):
        image = cv2.resize(self.image, self.size, cv2.INTER_LINEAR)
        _, encimg = cv2.imencode(".png", image)
        data_encode = np.array(encimg)
        self.bytes = io.BytesIO(data_encode.tobytes())
            