import threading
import time
import cv2
import numpy as np
import io
import requests
import os

try:
    from azure.cognitiveservices.vision.computervision import ComputerVisionClient
    from msrest.authentication import CognitiveServicesCredentials
except:
    pass
from algorithms.constants import WIDTH, HEIGHT


class CVMS:
    def __init__(
        self, enable, end_point, logger, interval=0, size=(WIDTH, HEIGHT)
    ):
        self.endpoint = end_point
        self.logger = logger
        self.reconfigure(enable)

        self.det_time_int = interval
        self.last_det_time = time.time()
        self.size = size
        self.image = np.zeros((self.size[0], self.size[1], 3), np.uint8)
        self.image = self.preprocess(self.image)
        self.result = []
        self.timeout = 2
        self.processing = threading.Lock()
        
        subscription_key = (
            open(f'{os.path.expanduser("~")}/.ms/key').readline().strip()
        )
        try:
            self.runtime = ComputerVisionClient(
                self.endpoint, CognitiveServicesCredentials(subscription_key)
            )
        except:
            self.runtime = None
            self.logger.warning("Microsoft Azure Computer Vision API Key not present, Cloud Features Disabled.")
            
    def reconfigure(self,enable):
        self.enable =  enable
        
    def get_objects(self):
        return self.result

    def set_image(self, image):
        self.image = image

    def set_enable(self, enable):
        if not enable:
            self.reset()
        self.enable = enable

    def reset(self):
        self.result = []

    def preprocess(self, image):
        image = cv2.resize(image, self.size, cv2.INTER_LINEAR)
        _, encimg = cv2.imencode(".png", image)
        data_encode = np.array(encimg)
        self.image = io.BytesIO(data_encode.tobytes())
        return self.image

    def detect(self, is_idle, is_connected):
        if not is_connected:
            self.reset()
        if not self.enable or not is_connected:
            return
        det_time = time.time()
        if (
            det_time - self.last_det_time > self.det_time_int
            and not is_idle
            and not self.processing.locked()
        ):
            self.last_det_time = det_time
            # only detect if there's movement between frames
            Thread1 = threading.Thread(target=self.infer_lock)
            Thread1.daemon = True
            Thread1.start()
            Thread1.join(0.0001)

    def infer_lock(self):
        with self.processing:
            self.infer()

    def draw(self, frame):
        pass
