try:
    from azure.cognitiveservices.vision.computervision.models import (
        OperationStatusCodes,
    )
except:
    pass

import numpy as np
import time
from .cv import CVMS


class SceneTextDetectionMS(CVMS):
    def infer(self):
        w, h = self.size
        try:
            read_response = self.runtime.read_in_stream(self.image, raw=True)
            read_operation_location = read_response.headers["Operation-Location"]
            operation_id = read_operation_location.split("/")[-1]

            # Call the "GET" API and wait for it to retrieve the results
            while True:
                try:
                    read_result = self.runtime.get_read_result(operation_id)
                    if read_result.status not in ["notStarted", "running"]:
                        break
                    time.sleep(0.05)
                except Exception as e:
                    return

            results = []
            if read_result.status != OperationStatusCodes.succeeded:
                return
            for text_result in read_result.analyze_result.read_results:
                for line in text_result.lines:
                    # normalize x with h, and y with w
                    x_cords = [i / h for i in line.bounding_box[::2]]
                    y_cords = [i / w for i in line.bounding_box[1::2]]
                    rect = np.array([*zip(x_cords, y_cords)])
                    pts = rect.reshape((-1, 1, 2))
                    text = filter_text(line.text)
                    if text != "":
                        results.append(dict(box=pts, text=text))
            self.result = results
        except Exception as e:
            self.logger.warning(f"Scene Text Detection Failed: {e}")


import re


def filter_text(text, remove_chinese=False):
    if remove_chinese:
        # Remove anything that is not a letter, space character, or only contains spaces
        return re.sub(r"[^a-zA-Z0-9 ]", "", text).strip()

    # Remove anything that is not a letter, space character, or Chinese character, or only contains spaces
    return re.sub(r"[^a-zA-Z0-9\u4e00-\u9fa5 ]", "", text).strip()


if __name__ == "__main__":
    # Example usage
    input_text = "Hello, 123 world! 你好，世界！🌍"
    filtered_text = filter_text(input_text)
    print(filtered_text)  # Output: "Hello world"
    input_text = "   "
    filtered_text = filter_text(input_text)
    print(filtered_text)  # Output: ""
