
from dataclasses import dataclass
from typing import Any
@dataclass
class ObstacleAvoidance:
    enable: bool
    model_file: str
    black_and_white: bool
    use_obstacle_detect: bool
    use_apf_route: bool
    alpha: float = 0.5
    
@dataclass
class ObjectTracking:
    enable: bool
    model_file: str
    use_tracking: bool
    
@dataclass
class ImagePathFollowing:
    enable: bool
    mock_route: bool
    mock_route_folder: str 
    use_superpoint: bool
    model_file: str
    
@dataclass
class MapPathFollowing:
    enable: bool
    mock_route: bool
    mock_route_folder: str 
    
@dataclass
class Cloud:
    image_captioning: bool
    image_tagging: bool
    scene_text_spotting: bool
    
@dataclass
class Visualize:
    enable: bool
    debug: bool
 
@dataclass
class TextToSpeech:
    enable: bool
    interval: float

@dataclass
class Config:
    namespace: str
    imu_namespace:  str
    camera0_namespace: str
    camera1_namespace: str
    rate: int
    engine_type: str
    obstacle_avoidance: ObstacleAvoidance
    object_tracking: ObjectTracking
    image_path_following: ImagePathFollowing
    map_path_following: MapPathFollowing
    cloud: Cloud
    visualize: Visualize
    text_to_speech: TextToSpeech


def find_param_str_in_config( config:Config, param_str:str) -> Any:
    '''
    param_str: str , for example, "obstacle_avoidance.enable"
    config: the config object
    return the value of the config, for example, ObstacleAvoidance.enable
    '''
    param_levels = param_str.split(".")
    config_value= config
    try:
        for param in param_levels:
            config_value = config_value.__getattribute__(param)
        return config_value
    except Exception as e:
        return None

def set_param_str_in_config(config:Config,  param_str:str, value:Any) -> bool:
    '''
    config_str: str , for example, "obstacle_avoidance.enable"
    value:  the value to set
    '''
    param_levels = param_str.split(".")
    config_value= config
    try:
        for param in param_levels[:-1]:
            config_value = config_value.__getattribute__(param)
        config_value.__setattr__(param_levels[-1], value)
        return True
    except Exception as e:
        raise e 

    
def sample_config() -> Config:
    config= Config(
        namespace="",
        imu_namespace="",
        camera0_namespace="",
        camera1_namespace="",
        rate=10,
        engine_type="dummy",
        obstacle_avoidance=ObstacleAvoidance(
            enable=True,
            model_file = '',
            black_and_white=False,
            use_obstacle_detect = False,
            use_apf_route = False,
            alpha=0.5,
        ),
        object_tracking=ObjectTracking(
            enable=True,
            model_file = '',
            use_tracking = False,
        ),
        image_path_following=ImagePathFollowing(
            enable=False,
            mock_route = False,
            mock_route_folder='',
            use_superpoint = False,
            model_file = '',
        ),
        map_path_following=MapPathFollowing(
            enable=False,
            mock_route = False,
            mock_route_folder='',
        ),
        cloud=Cloud(
            image_captioning=True,
            image_tagging=True,
            scene_text_spotting=True,
        ),
        visualize=Visualize(
            enable=True,
            debug=True,
        ),
        text_to_speech=TextToSpeech(
            enable=True,
            interval=0.5,
        ),
    )
    return config


if __name__ == '__main__':
    config = sample_config()
    print(config.obstacle_avoidance.enable, type(config.obstacle_avoidance.enable))
    config.obstacle_avoidance.enable = False
    print(config.obstacle_avoidance.enable, type(config.obstacle_avoidance.enable))
    
    print(find_param_str_in_config(config, "map_path_following.enable"))
    set_param_str_in_config(config, "map_path_following.enable", True)
    print(find_param_str_in_config(config, "map_path_following.enable"))