
import requests
import os

from algorithms.constants import MAIN_SERVER

class AuthProvider:
    def __init__(self, logger):
        self.header = {}
        self.device_info = {}
        
        self.logger = logger
        self.header = {
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }
        try:
            # file is stored in the home directory, under the folder .creds
            creds_file = open(os.path.join(
                os.path.expanduser('~'), '.creds', 'main_server'), 'r')
            creds = [line.strip() for line in creds_file.readlines()]
            creds_file.close()
            api_key ,device_id = creds
            
            self.header['Authorization'] = f"Bearer {api_key}"
            self.fetch_device_info(device_id)
        except:
            self.logger.info('~/.creds/main_server is not setup, unable to use online features')
        
    def fetch_device_info(self,device_id):
        '''
        Fetch device info from main server
        device_id: str
        
        device_info: dict(
            current_trip: int,
            starting_lat: float,
            starting_lng: float,
            ending_lat: float,
            ending_lng: float,
        )
        '''
        try:
            response = requests.get(
                f'{MAIN_SERVER}/device/list_device', 
                headers=self.header,
                params={'device_id':device_id},
                timeout=5
            ).json()
            
            self.device_info = response[device_id]  
        except Exception as e:
            self.logger.error(f"Main Server: Get Device Info Failed: {e}")
            return 
    
    def get_header_and_device_info(self):
        return self.header, self.device_info
    