# This class is used to check whether internet connection is established from time to time
import socket
from .utils import PeriodicThread


def ping_server(server: str, port: int, timeout=3):
    """ping a http server with ip and port"""
    # strips the http:// or https:// and :port from the server string
    server = server.split("//")[-1].split(":")[0]
    try:
        socket.setdefaulttimeout(timeout)
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((server, port))
    except OSError as error:
        return False
    else:
        s.close()
        return True


class InternetChecker:
    def __init__(self, logger):
        self.logger = logger
        self.is_connected = False

        self.thread = PeriodicThread(1, self.check_connect)
        
    def start(self):
        if self.thread and not self.thread.running:
            self.thread.start()
    def connected(self):
        return self.is_connected

    def stop(self):
        print("releasing internet checker")
        if self.thread and self.thread.running:
            self.thread.stop()
    def release(self):
        self.stop()

    def check_connect(self):
        has_been_connected = self.is_connected
        self.is_connected = ping_server("www.google.com", 80, timeout=3)
        if self.is_connected and not has_been_connected:
            self.logger.info(f"Established internet connection")
        elif not self.is_connected and has_been_connected:
            self.logger.info(f"Internet connection lost")
