import time
from algorithms.config import Config
from algorithms.constants import  HEIGHT, OBS_DIST_TOO_CLOSE_SCREEN_RATIO

class TextToSpeech:
    def __init__(self, config: Config):
        self.config = config
        self.text = ''
        self.reconfigure(config)
        self.last_speak_time = time.time()

    def reconfigure(self, config: Config):
        self.config = config

    def speak(
        self, 
        matcher_states,
        template_info,
        obs_dir, obs_dist,
        image_caption,
        image_tags, 
        text_detected
    ):
        if time.time() - self.last_speak_time < self.config.text_to_speech.interval or not self.config.text_to_speech.enable:
            return ''
        text = []    
        if matcher_states is not None:
            first_match = matcher_states['first_match']
            is_decision_point, keyframe_desc = template_info['is_decision_point'],template_info['keyframe_desc']
        
            if not first_match:
                text.append(f'Please go to the starting point')# 請前往起點')
            else:
                kf_text = []
                if is_decision_point:
                    kf_text.append(f'Decision Point: ')
                if keyframe_desc != '':
                    kf_text.append(f'{keyframe_desc}')
                text.append(' '.join(kf_text))


        # Obstacle Avoidance Voice Cue        
        if obs_dir is not None and obs_dist is not None:
            obs_text = []
            if obs_dir > 0:
                obs_text.append('Left')
            elif obs_dir < 0:
                obs_text.append('Right')
            if obs_dist < OBS_DIST_TOO_CLOSE_SCREEN_RATIO* HEIGHT:
                obs_text.append('Front')
            if len(obs_text)>0: 
                # only insert 'obstacle' when detected
                obs_text.append('Obstacle')
            
            text.append(' '.join(obs_text))
            
        if len(image_caption) > 0:
            text.append(f'In front is: {image_caption[0]["text"]}')
        if len(image_tags) > 0:
            text.append(f"Tags: {' '.join([st['name'] for st in image_tags])}")   
        if len(text_detected) > 0:
            text.append(f"Detected Text: {' '.join([st['text'] for st in text_detected])}")

        self.text = '. '.join(text)
        self.last_speak_time = time.time()
        return self.text
