import os
import numpy as np
import cv2
try:
    from hobot_dnn import pyeasy_dnn as dnn
except ImportError:
    print("Please install hobot_dnn first")    
    raise ImportError
from .keypoints import KeyPoint
from algorithms.constants import HEIGHT, WIDTH

class KeyPointBPU(KeyPoint):
    def __init__(self,root,config_loader):
        KeyPoint.__init__(self,root,config_loader)
        if not self.enable:
            return
        # get class 
        assert os.path.exists(self.engine_file), "Weights file not found"
        assert self.engine_file.endswith('.bin'), "file must be .bin"
        self.engine = dnn.load(self.engine_file)[0]
    
    def infer(self):       

        output = self.engine.forward(self.image)
        out1= output[0].buffer
        out2= output[1].buffer
        self.extract(np.array(out1), np.array(out2))

    def preprocess(self,image,size=(WIDTH, HEIGHT)):
        resized_image = cv2.resize(image, size, cv2.INTER_LINEAR).astype(np.uint8)

        # convert color
        if len(resized_image.shape) != 2:
            resized_image = cv2.cvtColor(resized_image, cv2.COLOR_BGR2GRAY).astype(np.uint8)
        self.image= resized_image[np.newaxis, np.newaxis,:, :]
        return True

    def destroy(self):
        pass
