from math import tan, atan, radians
import requests
import numpy as np
import cv2
import base64
import io
from algorithms.constants import  WIDTH, HEIGHT, CAMERA_RECEPTIVE_ANGLE

# HEIGHT = 192
# WIDTH = 320
# CAMERA_RECEPTIVE_ANGLE = 120


def valid_box(box):
    if len(box) == 0:
        return False
    if box[2]*box[3]*HEIGHT*WIDTH < 100:
        return False
    if box[2]*HEIGHT < 10 or box[3]*WIDTH < 10:
        return False
    return True


def xyxy2xywh_c(x1, y1, x2, y2):
    return (x1+x2)//2, (y1+y2)//2, abs(x2 - x1), abs(y2 - y1)


def screen_degree_diff(w1, w2):
    '''
    w1: the x coordinate of the self center
    w2: the x coordinate of the target x
    return the angle the user should turn to face the target
    '''
    angle_rad = atan((w2-w1)/(WIDTH/2/tan(radians(CAMERA_RECEPTIVE_ANGLE//2))))
    angle_deg = np.rad2deg(angle_rad)
    return angle_deg


def dir_to_face_object(obj, width):
    x, _, _, _ = obj
    center = width // 2
    dev_angle = screen_degree_diff(center, x)
    return dev_angle


def get_dir_from_boxes(boxes):
    '''
    boxes: [match_box, image_box, template_box]
    if the image box and template box coordinates are not on the same side of the image,
    return the direction the box should move so that the template box and image box are on the same side
    such that the user should move the opposite direction to face the target
    the direction for not moving is 0
    negative direction is left
    positive direction is right
    '''
    if len(boxes[0]) == 0:
        return 0
    image_box, template_box = boxes[1], boxes[2]
    image_box_center_w = (image_box[0]+image_box[2])//2
    template_box_center_w = (template_box[0]+template_box[2])//2
    angle_deg = screen_degree_diff(image_box_center_w, template_box_center_w)
    # this deg is what the box should move, therefore the camera should move the opposite direction
    angle_deg = -angle_deg
    return angle_deg


def relative_angle(me, target):
    """Returns the angle between two angles in degrees"""
    return (target - me + 180) % 360 - 180


def img2bytes(img):
    buf = cv2.imencode('.png', img)[1]
    data_encode = np.array(buf)
    byte = io.BytesIO(data_encode.tobytes())
    image_bytes = list(byte.getvalue())
    return image_bytes


def img2base64(img):
    buf = cv2.imencode('.png', img)[1].tobytes()
    buf = base64.b64encode(buf).decode('utf-8')
    return buf


def base642img(b64):
    bytes = base64.b64decode(b64)
    return cv2.imdecode(np.frombuffer(bytes, np.uint8), 0)


if __name__ == '__main__':

    test_angles = [0, 45, 90, 135, 180, 225, 270, 315]
    test_angles = [0, 45, 90, 135, 180, 225, 270, 315]

    def test_relative_angle():
        for me in test_angles:
            for target in test_angles:
                print("me: %d, target: %d, relative: %d" %
                      (me, target, relative_angle(me, target)))
    test_relative_angle()

    def test_dir_to_face_object():
        for x in range(0, 320, 10):
            print("x: %d, angle: %d" %
                  (x, dir_to_face_object((x, 0, 0, 0), 320)))

    test_dir_to_face_object()

    def test_get_dir_from_boxes():
        angle = get_dir_from_boxes([[], [0, 0, 0, 0], [0, 0, 0, 0]])
        print(angle)

        angle = get_dir_from_boxes(
            [[0, 0, 0, 0], [0, 0, 160, 192], [160, 0, 320, 192]])
        print(angle)

        angle = get_dir_from_boxes(
            [[0, 0, 0, 0], [160, 0, 320, 192], [0, 0, 160, 192]])
        print(angle)
    test_get_dir_from_boxes()
