# This is a class to simulate a person walking in a pre-specified GPS coordinates
import numpy as np
import os
from .utils import get_path_from_txt,coords_interpolate
from algorithms.general.utils import PeriodicThread
from algorithms.constants import WALK_PATH,SENSOR_MAP

class ArtificialWalk:
    def __init__(self, config_loader ,root , plan=1):
        self.enable = config_loader.mapPathFollowing.enable and config_loader.mapPathFollowing.simulate
        self.filename = os.path.join(root, WALK_PATH)
        self.plan =plan

        # Thread for traversing through the pre-specified coordinates
        self.thread = PeriodicThread(0.1, self.move)
        self.video_portion = 0

        # Index of the current coordinate
        self.path_idx = -1

        # The coordinates to walk in sequence
        self.sim_walked_path = get_path_from_txt(self.filename, plan=self.plan)
        self.keyframes = np.array([i[2] for i in self.sim_walked_path])

    # Get the instantaneous artificial coordinate
    def spoof_data(self, sensor_data):
        if not self.enable or self.path_idx <0:
            return sensor_data
        
        self.data = self.sim_walked_path[self.path_idx]
        dir= self.data[0]
        lat,lng =  self.data[1]
        if self.path_idx <len(self.sim_walked_path) - 1:
            cur_por  = self.data[2]
            next_por = self.sim_walked_path[self.path_idx+1][2]
            nlat,nlng = self.sim_walked_path[self.path_idx+1][1]
            lat,lng = coords_interpolate((lat,lng),(nlat,nlng),(self.video_portion-cur_por)/(next_por-cur_por))
        sensor_data[SENSOR_MAP['yaw']] = dir
        sensor_data[SENSOR_MAP['lat']] = lat
        sensor_data[SENSOR_MAP['lng']] = lng
        return sensor_data

    def update(self,video_portion):
        self.video_portion = video_portion
        
    def move(self):
        if self.path_idx < len(self.sim_walked_path) - 1:
            # print(self.video_portion  - self.keyframes)
            diff =self.keyframes - self.video_portion
            self.path_idx = np.argmin(abs(diff))

    def start_simulate(self):
        if not self.enable:return self
        if self.thread.running:
            print('Thread for artificial path is already running')
            return None
        
        self.thread.start()
        print('Started artificial path generation thread')
        return self
    
    def stop_simulate(self):
        if not self.enable:return
        if self.thread.running:
            self.thread.stop()
            print('Stopped artificial path generation thread')
            self.thread = None