
import numpy as np
import cv2
import os
try:
    from hobot_dnn import pyeasy_dnn as dnn
except ImportError:
    print("Please install hobot_dnn first")    
    raise ImportError

from algorithms.constants import HEIGHT, WIDTH,PATH_OBJECTS,COCO_CLASS,CONF_THRESHOLD,IOU_THRESHOLD
from .object_detection import ObjectDetection
from .nms import get_det_boxes

class ObjectDetectionBPU(ObjectDetection):
    def __init__(self,root,config_loader):
        ObjectDetection.__init__(self,root,config_loader)
        # get class 
        assert os.path.exists(self.engine_file), "Weights file not found"
        assert self.engine_file.endswith('.bin'), "file must be .bin"
        self.engine = dnn.load(self.engine_file)[0]
        
    def preprocess(self,image:np.ndarray,size=(WIDTH, HEIGHT)):
        image = cv2.resize(image, size, cv2.INTER_LINEAR)
        # convert color
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB).astype(np.float32)
        image= np.expand_dims(image, axis=0).transpose(0,3,1,2).astype(np.uint8)
        return image
    
    def detect(self, image):        
        output = self.engine.forward(image)
        ans = np.squeeze(output[0].buffer, axis=-1)
        result = get_det_boxes(ans,COCO_CLASS,CONF_THRESHOLD,IOU_THRESHOLD,max_det=10)
        return result
    
    def destroy(self):
        pass
