from math import sin, cos, tan, sqrt, atan2, atan, asin, fabs, acos, radians, degrees, pi, trunc
from algorithms.constants import WIDTH,CAMERA_RECEPTIVE_ANGLE

def xyxy2xywh_c(x1, y1, x2, y2):
    return (x1+x2)//2, (y1+y2)//2, abs(x2 - x1), abs(y2 - y1)

def bbox_iou(box1, box2):
    """Returns the IoU of two bounding boxes
    """
    # Get the coordinates of bounding boxes
    b1_x1, b1_y1, b1_w, b1_h = box1
    b2_x1, b2_y1, b2_w, b2_h = box2
    b1_x2 = b1_x1 + b1_w
    b1_y2 = b1_y1 + b1_h
    b2_x2 = b2_x1 + b2_w
    b2_y2 = b2_y1 + b2_h

    # get the corrdinates of the intersection rectangle
    inter_rect_x1 = max(b1_x1, b2_x1)
    inter_rect_y1 = max(b1_y1, b2_y1)
    inter_rect_x2 = min(b1_x2, b2_x2)
    inter_rect_y2 = min(b1_y2, b2_y2)
    # Intersection area
    inter_area = max(inter_rect_x2 - inter_rect_x1 + 1, 0) * \
        max(inter_rect_y2 - inter_rect_y1 + 1, 0)

    # Union Area
    b1_area = (b1_x2 - b1_x1 + 1) * (b1_y2 - b1_y1 + 1)
    b2_area = (b2_x2 - b2_x1 + 1) * (b2_y2 - b2_y1 + 1)

    iou = inter_area / (b1_area + b2_area - inter_area + 1e-16)

    return iou

def dir_to_face_object(obj):
    x, _,_,_= obj    
    off_center = x/(WIDTH//2) - 1
    dev_angle= atan(off_center* tan(radians(CAMERA_RECEPTIVE_ANGLE)//2)) * 57.295779513
    return dev_angle


def relative_angle(me, target):
    """Returns the angle between two angles in degrees"""
    return (target - me + 180) % 360 - 180
 