import cv2
import numpy as np
import torch
from algorithms.constants import HEIGHT, WIDTH

def get_binary_e2e(pr_mask):
    # Get binary mask from segmentation result
    binary_mask = get_binary(pr_mask)
    # Extract all contours including outside the main contour
    contours = get_contour(binary_mask)
    # Update binary mask with only masks overlapping with the largest mask
    binary_mask = update_binary(binary_mask, contours)
    # # Extract contour only inside the main contour
    # contours = get_contour(binary_mask)
    # # Update binary mask with only masks overlapping with the largest mask
    # binary_mask = update_binary_with_contours(binary_mask, contours)
    return binary_mask,contours

def get_binary(pr_mask):
    mask = pr_mask.squeeze()
    mask = np.array(mask*255, dtype=np.uint8)
    # mask = cv2.normalize(mask, None, 0, 255, cv2.NORM_MINMAX, cv2.CV_8U)
    # mask = np.expand_dims(mask,2)
    
    # blurred = cv2.GaussianBlur(mask, (3, 3), 0)
    # print(mask.shape)
    # blurred = cv2.medianBlur(mask, 5)
    # mask255 = np.array(cv2.normalize(blurred, None, 0, 255, cv2.NORM_MINMAX, cv2.CV_32F), dtype=np.uint8)
    # _, thresh = cv2.threshold(mask255, 180, 1, cv2.THRESH_BINARY)
    _, thresh = cv2.threshold(mask, 0, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)
    binary_mask = 1 - thresh

    # remove salt and pepper noise and smoothen the mask
    # to make sure the goal post is not an isolated point on the mask
    binary_mask = binary_mask.astype(np.uint8)
    
    # opening_kernel = np.ones((3,3), np.uint8)    

    # binary_mask = cv2.morphologyEx(binary_mask, cv2.MORPH_CLOSE, opening_kernel)

    # binary_mask = cv2.morphologyEx(binary_mask, cv2.MORPH_OPEN, opening_kernel)

    return binary_mask

def get_contour(binary_mask):
    # Extract contour from the binary mask
    res = cv2.findContours(255 * (1 - binary_mask), cv2.RETR_CCOMP, cv2.CHAIN_APPROX_SIMPLE)
    if len(res) == 3:
        _, cnts, hierarchy = res
    else:
        cnts, hierarchy = res
    cnts_area = []
    for cnt in cnts:
        cnts_area.append(cv2.contourArea(cnt))
    
    # No contour found, return default contour
    if len(cnts) == 0:
        return np.array([[[0, 0]], [[0, HEIGHT - 1]], [[WIDTH - 1, HEIGHT - 1]], [[WIDTH - 1, 0]]], dtype=np.int32)

    # Sort the contours and their hierarchy by area
    cnts, hierarchy, cnts_area = zip(*sorted(zip(cnts, hierarchy[0], cnts_area), key=lambda x : x[2], reverse=True))

    contours = []

    for i in range(len(cnts)):
        if cnts_area[i] > 300:
            contours.append(cnts[i])
            break
    
    return contours

def update_binary(binary_mask, contours):
    new_mask = np.zeros(binary_mask.shape, dtype=np.uint8)
    if len(contours) > 0:
        new_mask = cv2.drawContours(new_mask, [contours[0]], -1, 1, thickness=-1)

    new_mask = 1 - np.bitwise_and(new_mask, 1 - binary_mask)
    
    return new_mask

