import numpy as np
import time
from algorithms.constants import WIDTH, HEIGHT

class Goal:
    def __init__(self):
        self.prev_goal = None
        self.prev_time = time.time()
        
    def get_goal(self, binary_mask):
        # Naive goal extraction from the segmentation
        walkable_indices = np.asarray(np.where(binary_mask == 0))
        # Goal extraction
        if len(walkable_indices[0, :]) > 1:
            farthest_x, farthest_y = self.extract_goal_from_binary_mask(walkable_indices)
            new_x = np.clip(farthest_x, 0, WIDTH - 1)
            if new_x != farthest_x:
                farthest_x, farthest_y = self.reextract_goal_from_new_x(walkable_indices, new_x, farthest_x, farthest_y)

            # Restrict the maximum change of the goal
            farthest_x, farthest_y = self.restrict_goal(binary_mask, farthest_x, farthest_y)
        # Default goal in case no goal can be extracted
        else:
            farthest_x = WIDTH // 2
            farthest_y = 0

        goal = [int(farthest_x), int(farthest_y)]
        self.prev_goal = goal
        self.prev_time = time.time()
        return goal

    def restrict_goal(self, binary_mask,farthest_x, farthest_y):
        # Restrict the moving speed of the goal from one frame to another
        time_elapsed = time.time() - self.prev_time
        max_diff_left = int(time_elapsed * 165)
        max_diff_right = int(time_elapsed * 165)
        farthest_x = np.clip(farthest_x, 0, WIDTH - 1)

        # prev goal exists
        if self.prev_goal is not None:
            diff_x = farthest_x - self.prev_goal[0]
            diff_x = np.clip(diff_x, -max_diff_left, max_diff_right)
            new_x = self.prev_goal[0] + diff_x
            y_candidates = np.where(binary_mask[:, int(new_x)] == 0)[0]
            if len(y_candidates) > 0:
                new_y = np.min(y_candidates)
                return int(new_x), int(new_y)
        
        return farthest_x, farthest_y
    
    def extract_goal_from_binary_mask(self, walkable_indices):
        top_y = walkable_indices[0][0]
        bottom_y = top_y + 3
        ranged_id = np.asarray(np.where(walkable_indices[0, :] < bottom_y))

        start_coord = [WIDTH // 2, HEIGHT - 2]
        for i in ranged_id[0]:
            dist = np.linalg.norm((walkable_indices[1][i] - start_coord[0], walkable_indices[0][i] - start_coord[1]))

        percentile_idx = np.argmin(np.abs(np.percentile(dist, 70) - dist))

        farest_y = walkable_indices[0, ranged_id[0][percentile_idx]]
        farest_x = walkable_indices[1, ranged_id[0][percentile_idx]]

        return farest_x, farest_y
    
    def reextract_goal_from_new_x(self, walkable_indices, new_x, old_x, old_y):
        for i in range(new_x, old_x, (-5 if (new_x > old_x) else 5)):
            goal_line_id = np.asarray(np.where(walkable_indices[1, :] == i))
            if len(goal_line_id[0, :]) > 0:
                farest_x = i
                farest_y = walkable_indices[0, goal_line_id[0, :][0]]
                farest_y = (HEIGHT - 1) if ((farest_y + 3) > (HEIGHT - 1)) else (farest_y + 3)
                return farest_x, farest_y
        return old_x, old_y