
import numpy as np
import cv2
from os.path import join
from algorithms.constants import (
    HEIGHT,WIDTH,
    SEG_IMAGE_UNIFORM_THRES,
    SEG_PR_MASK_MEAN_THRES,
)
from algorithms.config import Config
    
class Segmentation:
    def __init__(self, root, config_loader:Config):
        self.config = config_loader
        # self.image= np.zeros((1,3,HEIGHT,WIDTH),np.float32) if not bw else np.zeros((1,1,HEIGHT,WIDTH),np.float32)
        self.seg= np.zeros((HEIGHT,WIDTH,1),np.float32)
        self.memory=  np.zeros((HEIGHT,WIDTH,1),np.float32)
        
        self.engine_file = join(root, config_loader.obstacle_avoidance.model_file)      
        self.engine = None
    
    def preprocess(self,image,size=(WIDTH, HEIGHT)):
        # shape: [H, W, 3] if not bw else [H, W]
        resized_image = cv2.resize(image, size, cv2.INTER_LINEAR)

        if self.config.obstacle_avoidance.black_and_white:
            image = cv2.cvtColor(resized_image, cv2.COLOR_BGR2GRAY)
            # shape: [H, W, 1]
            image =  np.expand_dims(image, axis=2)
            MEAN = 255 * np.mean([0.485, 0.456, 0.406])
            STD = 255 * np.mean([0.229, 0.224, 0.225])
            image = (image - MEAN) / STD
        else:
            image = cv2.cvtColor(resized_image, cv2.COLOR_BGR2RGB)
            # image = image/255.0
            MEAN = 255 * np.array([0.485, 0.456, 0.406])
            STD = 255 * np.array([0.229, 0.224, 0.225])
            image = (image - MEAN[ None, None,:]) / STD[None, None,:]

        # shape: [1, 1 or 3, H, W]   
        image= np.expand_dims(image, axis=0).transpose(0,3,1,2).astype(np.float32)     
        
        if self.image_is_bad(resized_image):
            return image, False
        return image, True

    def infer(self, image):     
        seg = self.infer_seg(image)
        
        alpha = self.config.obstacle_avoidance.alpha   
        self.memory= alpha*seg+(1-alpha)*self.memory
        # cv2.imshow('seg',self.memory)
        if np.mean(self.memory)<SEG_PR_MASK_MEAN_THRES:
            return self.memory,False
        return self.memory,True
    
    def infer_seg(self,image):
        pass

    def image_is_bad(self,image):
        image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        image =cv2.normalize(image, None, 0, 1, cv2.NORM_MINMAX, cv2.CV_32F)
        blur1_uni = cv2.GaussianBlur(image, (7, 7), 1)
        blur2_uni = cv2.GaussianBlur(image, (31, 31), 2)
        ssd_blur_uni = np.sum((blur1_uni - blur2_uni)**2)
        if  ssd_blur_uni<SEG_IMAGE_UNIFORM_THRES:
            return True
        return False

    def destroy(self):
        pass