
import os
import numpy as np
import cv2
try:
    from hobot_dnn import pyeasy_dnn as dnn
except ImportError:
    print("Please install hobot_dnn first")    
    raise ImportError

from algorithms.constants import HEIGHT, WIDTH 
from .segmentation import Segmentation

class SegmentationBPU(Segmentation):
    def __init__(self,root, config_loader):
        Segmentation.__init__(self,root, config_loader)
        assert os.path.exists(self.engine_file), "Weights file not found"
        assert self.engine_file.endswith('.bin'), "file must be .bin"
        self.engine = dnn.load(self.engine_file)[0]

    def infer_seg(self, image:np.ndarray):
        output = self.engine.forward(image)[0].buffer
        output = np.array(output).squeeze()
        seg = np.expand_dims(output,-1)
        return seg
    

    def preprocess(self, im, size=(WIDTH, HEIGHT)):
        resized_image = cv2.resize(im, size, cv2.INTER_LINEAR)
        # convert color
        image = cv2.cvtColor(resized_image, cv2.COLOR_BGR2RGB)
        image= np.expand_dims(image, axis=0).transpose(0,3,1,2).astype(np.uint8)

        return image, True
    
    def destroy(self):
        pass