
import os 
import numpy as np
try:
    from onnxruntime import InferenceSession
except:
    pass
from .segmentation import Segmentation

class SegmentationONNX(Segmentation):
    def __init__(self,root, config_loader):
        Segmentation.__init__(self,root, config_loader)
        assert os.path.exists(self.engine_file), "Weights file not found"
        assert self.engine_file.endswith('.onnx'), "file must be .onnx"
            
        self.engine = InferenceSession(self.engine_file, providers=['CPUExecutionProvider'])

    def infer_seg(self, image:np.ndarray):
        output = np.array(self.engine.run(['output'], {"images": image.astype(np.float32)})[0]).squeeze()
        seg = np.expand_dims(output,-1)
        return seg
        
    def destroy(self):
        pass