import numpy as np

from algorithms.constants import HEIGHT, WIDTH,OBS_TURN_THRESHOLD


### Returns the direction to go, from -1 to 1 with decimals.
### 0 = straight, -1 = left, 1 = right
### Obtain the direction based on the angle between starting coord and goal coord
def get_obs_direction(turning_point, turn_queue ):

    direction = get_direction((WIDTH // 2,HEIGHT - 1),turning_point)
    
    if turn_queue.full():
        turn_queue.get()
    turn_queue.put(direction)

    return get_avg_direction(turn_queue)

def get_direction(start , end):
    start_x , start_y = start
    x,y = end
    y_diff = y - start_y
    x_diff = x - start_x

    turn_thresh = OBS_TURN_THRESHOLD
    direction = 0
    if (y_diff != 0) or (x_diff != 0):
        angle = 90 + (np.arctan2(y_diff, x_diff) * 180 / np.pi)
        angle = np.clip(angle, -90, 90)

        ### turning left
        if angle < -turn_thresh:
            direction = (angle + turn_thresh) / (90. - turn_thresh)
            pass
        ### turning right
        elif angle > turn_thresh:
            direction = (angle - turn_thresh) / (90. - turn_thresh)
            pass
        ### going straight
        else:
            direction = 0
            pass
    return direction

def get_avg_direction( turn_queue):
    '''
    For the last n turns, count the number of left, right and straight turns.
    Return the direction with the most number of turns and the average of that direction.
    ''' 

    turn_list = np.array(turn_queue.queue)
    _num = [0, 0, 0]
    for i in range(len(turn_list)):
        if turn_list[i] < 0:
            _num[0] += 1
        elif turn_list[i] > 0:
            _num[2] += 1
        else:
            _num[1] += 1

    max_idx = np.argmax(_num)

    sum_of_item = 0
    num_of_item = _num[max_idx]

    if max_idx == 0:
        sum_of_item = turn_list[np.where(turn_list < 0)[0]].sum()
    elif max_idx == 1:
        sum_of_item = turn_list[np.where(turn_list ==0)[0]].sum()
    elif max_idx == 2:
        sum_of_item = turn_list[np.where(turn_list > 0)[0]].sum()

    _avg = (sum_of_item / num_of_item)

    _avg = np.clip(_avg,-1,1)
    
    return _avg

# Calculating distance between straight line generated with p1 & p2 and point p3
def pl_dist(line, point):
    p1 = line[0]
    p2 = line[1]
    p3 = point
    d = np.abs(
        ((p2[0] - p1[0]) * (p1[1] - p3[1]) 
         - (p1[0] - p3[0]) * (p2[1] - p1[1]))
        / np.sqrt((p2[0] - p1[0]) ** 2
                  + (p2[1] - p1[1]) ** 2)
    )
    return d